/**
 * 
 */
package org.springblade.datasource.enums;

import java.util.function.Function;

import org.springblade.report.entity.ReportDatasource;

import com.baomidou.mybatisplus.annotation.EnumValue;

import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author wookvn
 *	数据源
 */
@Getter
@AllArgsConstructor
public enum DsTypeEnum {

	/**
	 * zeroDateTimeBehavior=CONVERT_TO_NULL - 解决java.sql.SQLException: Zero date value prohibited 
	 * allowPublicKeyRetrieval=true - 解决mysql8.0 Public Key Retrieval is not allowed
	 */
	mysql(0, "com.mysql.cj.jdbc.Driver", "select 1", 3000, 1000000,
			args -> String.format("select count(*) from (select 1 from %s limit %s) a", args), 
			ds -> String.format("jdbc:mysql://%s:%s/%s", ds.getDsIp(), ds.getDsPort(), ds.getDsDb()) + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=CONVERT_TO_NULL&rewriteBatchedStatements=true&allowPublicKeyRetrieval=true&serverTimezone=GMT%2B8"),
	
	oracle(1, "oracle.jdbc.OracleDriver", "select 1 from dual", 5000, 1000000,
			args -> String.format("SELECT count(1) FROM %s WHERE ROWNUM <= %s", args), 
			ds -> String.format("jdbc:oracle:thin:@%s:%s/%s", ds.getDsIp(), ds.getDsPort(), ds.getDsDb())),
	
	sqlserver(2, "com.microsoft.sqlserver.jdbc.SQLServerDriver", "select 1", 3000, 1000000,
			args -> String.format("SELECT rows FROM sysindexes WHERE id = OBJECT_ID('%s') AND indid < 2", args), 
			ds -> String.format("jdbc:sqlserver://%s:%s;DatabaseName=%s", ds.getDsIp(), ds.getDsPort(), ds.getDsDb())),
	
	postgresql(3, "org.postgresql.Driver", "select 1", 3000, 1000000,
			args -> String.format("select count(*) from (select 1 from %s limit %s) a", args), 
			ds -> String.format("jdbc:postgresql://%s:%s/%s?currentSchema=%s", ds.getDsIp(), ds.getDsPort(), ds.getDsDb(), ds.getDbSchema())),
	
	clickhouse(4, "com.clickhouse.jdbc.ClickHouseDriver", "select 1", 50000, 100000000,
			args -> String.format("select count(1) from %s", args), 
			ds -> String.format("jdbc:clickhouse://%s:%s/%s", ds.getDsIp(), ds.getDsPort(), ds.getDsDb())),
	
	kingbase(5, "com.kingbase8.Driver", "select 1", 3000, 1000000, 
			args -> String.format("select count(*) from (select 1 from %s limit %s) a", args), 
			ds -> String.format("jdbc:kingbase8://%s:%s/%s?currentSchema=%s", ds.getDsIp(), ds.getDsPort(), ds.getDsDb(), ds.getDbSchema())),
	
	dameng(6, "dm.jdbc.driver.DmDriver", "select 1", 3000, 1000000, 
			args -> String.format("select count(*) from (select 1 from %s limit %s) a", args), 
			ds -> String.format("jdbc:dm://%s:%s/%s?schema=%s&zeroDateTimeBehavior=convertToNull&useUnicode=true&characterEncoding=utf-8", ds.getDsIp(), ds.getDsPort(), ds.getDsDb(), ds.getDbSchema())),
	
	//通用版本
	common(-1, "common", "common", 0, 0, args -> null, t -> null);
	
	@EnumValue
	private Integer value;
	private String driverClassName;
	private String validationQuery;
	private int batchSize;  //批量入库数量
	private int selectLimitVal; //查询最大限制数量
	private Function<Object[], String> checkLimitValueFunction; //校验数据量大小函数
	private Function<ReportDatasource, String> driverUrlFunction;

	/**
	 * 解析
	 * @param value
	 * @return
	 */
	public static DsTypeEnum resolve(Integer value) {
		DsTypeEnum[] values = DsTypeEnum.values();
		for(DsTypeEnum dsType : values) {
			if(value != null && dsType.getValue().intValue() == value) {
				return dsType;
			}
		}
		return DsTypeEnum.mysql;
	}
}
