/**
 * 
 */
package org.springblade.visual.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.cube.dto.PreviewDetailResult;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterCondition;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.enums.CommonFunctionEnum;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.VisualDataPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.microsoft.sqlserver.jdbc.StringUtils;

import cn.hutool.core.collection.ListUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("visual/component")
@Api(value = "模版组件", tags = "模版组件接口")
public class VisualComponentCotroller extends BladeController{

	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private DynamicTableService dynamicTableService;
	
	@Autowired
	private QuerySqlService querySqlService;
	
	@Autowired
	private VisualDataPermissionService visualDataPermissionService;
	
	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入visualComponent")
	public R<VisualComponent> detail(VisualComponent visualComponent) {
		VisualComponent detail = visualComponentService.getOne(Condition.getQueryWrapper(visualComponent));
		return R.data(detail);
	}

	/**
	 * 根据大屏id查询组件列表
	 * @param visualId
	 * @return
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "查询组件列表", notes = "传入大屏id visualId")
	public R<List<VisualComponent>> queryComponentListByVisualId(Integer visualId){

		return R.data(visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId,visualId)));
	}
	
	/**
	 * 详情
	 */
	@PostMapping("/delete")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除", notes = "传入ids")
	public R<String> delete(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(visualComponentService.removeByIds(Func.toIntList(ids)));
	}
	
	/**
	 * 
	 * @param value
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/detailPreview")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "清单组件数据预览", notes = "传入ComponentDto")
	public R<PreviewDetailResult> detailPreview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);
		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		VisualComponent component = componentDto.getComponent();
		R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, getUser(), ComponentTypeEnum.DETAIL);
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		QuerySqlParam querySqlParam = r.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		//页码和每页条数
		int pageNum = componentDto.getPageNum() == null || componentDto.getPageNum() <= 0 ? 1 : componentDto.getPageNum();
		int limitVal = component.getLimitVal() == null || component.getLimitVal() <= 0 ? CubeConstants.DEFAULT_PREVIEW_DATA_SIZE : component.getLimitVal(); 
		IPage<Map<String, Object>> pageData = dynamicTableService.selectPage(new Page<>(pageNum, limitVal), reportDatasource, querySql);
		PreviewDetailResult previewDetailResult = null;
		if(pageData != null) {
			List<Map<String, Object>> resultList = pageData.getRecords();
			//列权限，删除掉不显示的列
			_formatExcludeColumn(fieldDict, resultList, selectColumnList, querySqlParam.getExcludeColumnIdList());
			//封装数据
			previewDetailResult = new PreviewDetailResult(pageData.getTotal(), pageData.getPages(), pageData.getSize());
			previewDetailResult.setDataList(resultList);
			previewDetailResult.setFieldDict(fieldDict);
			previewDetailResult.setCubeName(querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null);
			previewDetailResult.setPermStatus(true);
		}
		return R.data(previewDetailResult, querySql);
	}
	
	/**
	 * 
	 * @param value
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/preview")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "数据预览", notes = "传入ComponentDto")
	public R<PreviewResult> preview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);
		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		VisualComponent component = componentDto.getComponent();
		R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, getUser());
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		QuerySqlParam querySqlParam = r.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		String querySql = querySqlParam.getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		List<Map<String, Object>> resultList = null;//dynamicTableService.selectList(reportDatasource, querySql);
		if (component.getLimitVal() != null && component.getLimitVal() > 0) { // 分页查询
			resultList = dynamicTableService.selectPageList(new Page<>(0, component.getLimitVal()).setSearchCount(false),
					reportDatasource, querySql);
		} else {
			resultList = dynamicTableService.selectList(reportDatasource, querySql);
		}
		//列权限，删除掉不显示的列
		_formatExcludeColumn(fieldDict, resultList, selectColumnList, querySqlParam.getExcludeColumnIdList());
		String cubeName = querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null;
		return R.data(new PreviewResult(resultList, fieldDict, cubeName), querySql);
	}
	
	/**
	 * 带分页的报表查询
	 * @param value
	 * @return
	 * @throws Exception
	 */
	@PostMapping("/pagePreview")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "分页报表预览", notes = "传入ComponentDto")
	public R<PreviewDetailResult> dynamicDetailPreview(@RequestBody String value) throws Exception{
		String decoderStr = new String(Base64.getDecoder().decode(value), "utf-8");
		ComponentDto componentDto = JSON.parseObject(decoderStr, ComponentDto.class);
		if(StringUtils.isEmpty(componentDto.getColumnList())) {
			return R.fail("字段列表不能为空");
		}
		if(componentDto.getVisualDataId() != null) {
			//校验大屏是否有访问权限
			if(!visualDataPermissionService.checkVisualDataForUser(componentDto.getVisualDataId(), componentDto.getVisualDataCreateUserId(), getUser())) {
				return R.fail("该大屏没有访问权限");
			}
		}
		//1.先根据维度数据聚合查询分页记录
		VisualComponent component = componentDto.getComponent();
		R<QuerySqlParam> logicPageQuerySqlRet = querySqlService.parseLogicPageQuerySql(component.getCubeDataId(), componentDto, getUser());
		if(!logicPageQuerySqlRet.isSuccess()) {
			return R.fail(logicPageQuerySqlRet.getMsg());
		}
		QuerySqlParam querySqlParam = logicPageQuerySqlRet.getData();
		ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
		CubeData cubeData = querySqlParam.getCubeData();
		Map<String, ComponentColumn> weiduColumnMap = querySqlParam.getSelectColumnList().stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, t -> t));
		String logicPageQuerySql = querySqlParam.getQuerySql();
		//维度聚合，查询出所有数据，做逻辑分页
		List<Map<String, Object>> logicDataList = dynamicTableService.selectList(reportDatasource, logicPageQuerySql);
		//页码和每页条数
		int pageNum = componentDto.getPageNum() == null || componentDto.getPageNum() <= 0 ? 1 : componentDto.getPageNum();
		int limitVal = component.getLimitVal() == null || component.getLimitVal() <= 0 ? CubeConstants.DEFAULT_PREVIEW_DATA_SIZE : component.getLimitVal();
		long total = logicDataList.size(), pages = (total + limitVal - 1) / limitVal;
		//取出当前页的维度数据
		List<Map<String, Object>> subDataList = ListUtil.sub(logicDataList, (pageNum - 1) * limitVal, pageNum * limitVal);
		//如果不为空，构造维度数据的where条件
		if(CollectionUtil.isNotEmpty(subDataList)) {
			Map<String, Set<Object>> valueMap = new HashMap<>();
			for(Map<String, Object> dataMap : subDataList) {
				dataMap.forEach((k,v) -> {
					Set<Object> objectSet = valueMap.get(k);
					if(objectSet == null) {
						objectSet = new HashSet<>();
						valueMap.put(k, objectSet);
					}
					objectSet.add(v);
				});
			}
			List<ComponentFilter> filterList = new ArrayList<ComponentFilter>();
			//构造时间字段过滤器
			valueMap.forEach((k,v) -> {
				ComponentFilter timeColumnFilter = new ComponentFilter(weiduColumnMap.get(k));
				timeColumnFilter.setConditions(Arrays.asList(new ComponentFilterCondition(CommonFunctionEnum.in, new ArrayList<>(v))));
				filterList.add(timeColumnFilter);
			});
			componentDto.setExtFilterJson(JSON.toJSONString(filterList));
		}
		R<QuerySqlParam> r = querySqlService.parseQuerySql(cubeData, reportDatasource, componentDto, getUser(), ComponentTypeEnum.GENERAL);
		if(!r.isSuccess()) {
			return R.fail(r.getMsg());
		}
		String querySql = r.getData().getQuerySql();
		List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
		Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
		List<Map<String, Object>> resultList = dynamicTableService.selectList(reportDatasource, querySql);
		PreviewDetailResult previewDetailResult = null;
		if(CollectionUtil.isNotEmpty(resultList)) {
			//列权限，删除掉不显示的列
			_formatExcludeColumn(fieldDict, resultList, selectColumnList, querySqlParam.getExcludeColumnIdList());
			previewDetailResult = new PreviewDetailResult(total, pages, limitVal);
			previewDetailResult.setDataList(resultList);
			previewDetailResult.setFieldDict(fieldDict);
			previewDetailResult.setCubeName(querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null);
			previewDetailResult.setPermStatus(true);
		}
		return R.data(previewDetailResult, querySql);
	}
	
	/**
	 * 列权限校验
	 * @param fieldDict
	 * @param resultList
	 * @param columnList
	 * @param excludeColumnIdList
	 */
	void _formatExcludeColumn(Map<String, String> fieldDict, List<Map<String, Object>> resultList, List<ComponentColumn> columnList, Set<Integer> excludeColumnIdList) {
		//列权限，删除掉不显示的列
		if(CollectionUtil.isNotEmpty(excludeColumnIdList) && CollectionUtil.isNotEmpty(resultList)) {
			Map<Integer, String> fieldIdMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, ComponentColumn::getColumnAliasName));
			List<String> excludeColumnAliasList = excludeColumnIdList.stream().map(it -> fieldIdMap.get(it)).filter(Objects::nonNull).collect(Collectors.toList());
			if(CollectionUtil.isNotEmpty(excludeColumnAliasList)) {
				//1.格式化数据
				for(Map<String, Object> jsonObj : resultList) {
					excludeColumnAliasList.stream().forEach(item -> jsonObj.remove(item));
				}
				if(fieldDict != null) { //2.格式化fieldDict
					excludeColumnAliasList.stream().forEach(item -> fieldDict.remove(item));
				}
			}
		}
	}
}

