/**
 * 
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.VisualDto;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualData;
import org.springblade.visual.entity.VisualDataPermission;
import org.springblade.visual.entity.VisualSelfReportPermission;
import org.springblade.visual.enums.VisualTypeEnum;
import org.springblade.visual.mapper.VisualDataMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.collection.CollectionUtil;

/**
 * @author wookvn
 *
 */
@Service
public class VisualDataService extends ServiceImpl<VisualDataMapper, VisualData> {

	@Autowired
	private VisualComponentService visualComponentService;
	
	@Autowired
	private VisualDataPermissionService visualDataPermissionService;

	@Autowired
	private VisualSelfReportPermissionService visualSelfReportPermissionService;

	/**
	 * 
	 * @param userId
	 * @return
	 */
	public List<Integer> classifyIdListPerm(Long userId,Integer visualDataType){
		return baseMapper.classifyIdListPerm(userId,visualDataType);
	}
	/**
	 * 
	 * @return
	 */
	@Transactional
	public Integer doSaveOrUpdate(VisualDto visualDto, BladeUser bladeUser){
		VisualData visualData = visualDto.getVisualData();
		Long userId = SecureUtil.getUserId();
		String deptId = SecureUtil.getDeptId();
		if(visualData.getId() == null) { //新增
			visualData.setCreateTime(LocalDateTime.now());
			visualData.setCreateUser(userId);
			visualData.setCreateDept(Func.toLong(deptId, 0L));
		}else {
			visualData.setUpdateTime(LocalDateTime.now());
			visualData.setUpdateUser(userId);
		}
		//1.新增或修改
		boolean ret = this.saveOrUpdate(visualData);
		if(ret) {
			List<ComponentDto> componentList = visualDto.getComponentList();
			//2.新增组件和组件字段
			visualComponentService.batchSaveComponent(visualData.getId(), componentList, bladeUser);
			
			//3.仅添加大屏时：权限组处理【前端在编辑时，不显示权限组选择】
			if(CollectionUtil.isNotEmpty(visualDto.getDataPermIdList())) {
				// 保存公共大屏与权限组的关系
				if (visualDto.getVisualData().getVisualDataType() == 0) {
					List<VisualDataPermission> dataList = new ArrayList<>(visualDto.getDataPermIdList().size());
					visualDto.getDataPermIdList().forEach(t -> {
						VisualDataPermission entity = new VisualDataPermission();
						entity.setCreateBy(bladeUser.getNickName());
						entity.setCreateTime(new Date());
						entity.setCreateUserId(bladeUser.getUserId());
						entity.setDataPermId(t);
						entity.setVisualId(visualData.getId());
						dataList.add(entity);
					});
					visualDataPermissionService.saveBatch(dataList);
				} else {
					// 保存自助报表与权限组的关系
					List<VisualSelfReportPermission> dataList = new ArrayList<>(visualDto.getDataPermIdList().size());
					visualDto.getDataPermIdList().forEach(t -> {
						VisualSelfReportPermission entity = new VisualSelfReportPermission();
						entity.setCreateBy(bladeUser.getNickName());
						entity.setCreateTime(new Date());
						entity.setCreateUserId(bladeUser.getUserId());
						entity.setDataPermId(t);
						entity.setSelfReportId(visualData.getId());
						dataList.add(entity);
					});
					visualSelfReportPermissionService.saveBatch(dataList);
				}
			}
		}
		return visualData.getId();
	}
	
	/**
	 * 删除
	 * @param idList
	 */
	@Transactional
	public void doRemove(List<Integer> idList) {
		baseMapper.deleteBatchIds(idList);
		visualComponentService.remove(Wrappers.<VisualComponent>lambdaQuery().in(VisualComponent::getVisualId, idList));
		List<VisualData> visualData = baseMapper.selectBatchIds(idList);
		Map<Integer, List<VisualData>> visualDataMap = visualData.stream().collect(Collectors.groupingBy(VisualData::getVisualDataType));
		if (CollectionUtil.isNotEmpty(visualDataMap)){
			if (CollectionUtil.isNotEmpty(visualDataMap.get(0))){
				// 删除大屏与权限组的关系
				List<Integer> ids = visualDataMap.get(0).stream().map(VisualData::getId).collect(Collectors.toList());
				visualDataPermissionService.remove(Wrappers.<VisualDataPermission>lambdaQuery().in(VisualDataPermission::getVisualId, ids));
			} else if (CollectionUtil.isNotEmpty(visualDataMap.get(1))){
				// 删除自助报表与权限组的关系
				List<Integer> ids = visualDataMap.get(1).stream().map(VisualData::getId).collect(Collectors.toList());
				visualSelfReportPermissionService.remove(Wrappers.<VisualSelfReportPermission>lambdaQuery().in(VisualSelfReportPermission::getSelfReportId,ids));
			}
		}

	}
	
	/**
	 * 复制幻灯片
	 * @param bladeUser
	 * @return <旧大屏ID, 新大屏ID>
	 */
	public List<Integer> doCopyPowerPoint(Set<Integer> visualIdSet, Integer powerPointId, BladeUser bladeUser){
		return doCopy(visualIdSet, null, null, powerPointId, VisualTypeEnum.POWER_POINT, bladeUser, "");
	}
	
	/**
	 * 复制报告
	 * @param reportInfoId
	 * @param bladeUser
	 * @return <旧大屏ID, 新大屏ID>
	 */
	public List<Integer> doCopyReport(Set<Integer> visualIdSet, Integer reportInfoId, BladeUser bladeUser) {
		return doCopy(visualIdSet, null, reportInfoId, null, VisualTypeEnum.REPORT, bladeUser, "");
	}
	
	/**
	 * @param visualIdSet      - 原大屏ID不能有重复
	 * @param visualCategoryId
	 * @param reportInfoId
	 * @param powerPointId
	 * @param visualType
	 * @param bladeUser
	 * @return <新大屏ID列表>
	 */
	@Transactional
	public List<Integer> doCopy(Set<Integer> visualIdSet, Integer visualCategoryId, Integer reportInfoId,
								Integer powerPointId, VisualTypeEnum visualType, BladeUser bladeUser, String titlePrefix) {
		List<Integer> visualIdList = new ArrayList<>();
		//批量添加大屏
		List<VisualData> visualDataList = baseMapper.selectBatchIds(visualIdSet);
		List<VisualComponent> visualComponentList = new ArrayList<VisualComponent>();
		visualDataList.forEach(item -> {
			final int oldVisualId = item.getId();
			item.setId(null);
			item.setTitle(titlePrefix + item.getTitle());
			item.setRefVisualId(oldVisualId);
			item.setVisualType(visualType);
			item.setCreateTime(LocalDateTime.now());
			item.setHistoryReportId(reportInfoId);
			item.setCreateUser(bladeUser.getUserId());
			item.setCreateDept(Func.toLong(bladeUser.getDeptId()));
			item.setUpdateTime(LocalDateTime.now());
			item.setUpdateUser(bladeUser.getUserId());
			item.setOrderIndex(9999);
			if(visualCategoryId != null) {
				item.setCategory(visualCategoryId);
			}
			if(powerPointId != null) {
				item.setPowerPointId(powerPointId);
			}
			int ret = baseMapper.insert(item);
			if(ret > 0) { //插入成功
				visualIdList.add(item.getId());
				//查询旧组件列表
				List<VisualComponent> componetList = visualComponentService.list(Wrappers.<VisualComponent>lambdaQuery().eq(VisualComponent::getVisualId, oldVisualId));
				if(componetList != null && !componetList.isEmpty()) {
					componetList.stream().forEach(f -> {
						f.setId(null);
						f.setVisualId(item.getId());
						f.setCreateBy(bladeUser.getNickName());
						f.setCreateTime(new Date());
						f.setCreateUserId(bladeUser.getUserId());
					});
					visualComponentList.addAll(componetList);
				}
			}
		});
		//批量添加组件列表
		if(CollectionUtil.isNotEmpty(visualComponentList)) {
			visualComponentService.saveBatch(visualComponentList);
		}
		return visualIdList;
	}
}
